package com.loafle.overflow.crawler.snmp;

import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.crawler.config.Config;
import com.loafle.overflow.crawler.config.Connection;
import com.loafle.overflow.crawler.config.Item;
import com.loafle.overflow.crawler.config.Query;
import com.loafle.overflow.crawler.snmp.version.SNMPv2c;
import com.loafle.overflow.crawler.snmp.version.SNMPv3;
import org.snmp4j.Snmp;
import org.snmp4j.TransportMapping;
import org.snmp4j.smi.Address;
import org.snmp4j.transport.DefaultUdpTransportMapping;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by insanity on 17. 4. 10.
 */
public class SNMPCrawler extends Crawler {

    @Override
    public Object getInternal(Config config) {

        Object retObj = null;
        String targetVer = (String)config.getTarget().getAuth().get("version");
        switch (targetVer) {
            case "v2c":
                retObj = getByV2c(config);
                break;
            case "v3":
                retObj = getByV3(config);
                break;
            default:
                new Exception("Unknown SNMP protocol : " + targetVer).printStackTrace();
        }

        Map m = (Map<String, String>)retObj;

        Map<String, String> keyMap = new LinkedHashMap<>();
        List<String> metrics = config.getItems().get(0).getMetrics();
        int i = 0;
        for (Object key : m.keySet()) {
            keyMap.put(metrics.get(i), (String)m.get(key));
            i++;
        }

        return keyMap;
    }

    private Object getByV2c(Config config) {
        Snmp snmp = null;
        TransportMapping<? extends Address> transport = null;
        try {
            transport = new DefaultUdpTransportMapping();
            snmp = new Snmp(transport);
            transport.listen();
            SNMPv2c snmpV2 = new SNMPv2c(snmp);

            Map<String, Object> authInfo = config.getTarget().getAuth();
            Connection connInfo = config.getTarget().getConnection();
            List<Item> items = config.getItems();
            String[] oids = new String[items.get(0).getQueries().size()];
            for (Item item : items) {
                List<Query> queries = item.getQueries();
                for (int i=0; i<queries.size();i++) {
                    oids[i] = queries.get(i).getQuery();
                }
            }

            String ip = connInfo.getIp();
            String port = connInfo.getPort();
            String community = (String)authInfo.get("community");
            return snmpV2.get(ip, port, community, oids);

            /*
            switch (method) {
                case "validate":
                    return snmpV2.validate(ip, port, community);
                case "get":
                    return snmpV2.get(ip, port, community, oids);
                case "walk":
                    return snmpV2.walk(ip, port, community, oids[0]);
                default:
                    new Exception("Unknown SNMP command : " + method).printStackTrace();
            }
            */

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (snmp != null) {
                try {
                    snmp.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (transport != null) {
                try {
                    transport.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    private Object getByV3(Config config) {
        Snmp snmp = null;
        TransportMapping<? extends Address> transport = null;

        try {
            transport = new DefaultUdpTransportMapping();
            snmp = new Snmp(transport);
            transport.listen();
            SNMPv3 snmpV3 = new SNMPv3(snmp);


            Map<String, Object> authInfo = config.getTarget().getAuth();
            Connection connInfo = config.getTarget().getConnection();
            List<Item> items = config.getItems();
            String[] oids = new String[items.get(0).getQueries().size()];
            for (Item item : items) {
                List<Query> queries = item.getQueries();
                for (int i=0; i<queries.size();i++) {
                    oids[i] = queries.get(i).getQuery();
                }
            }

            String ip = connInfo.getIp();
            String port = connInfo.getPort();
            String method = "get";

            String user = (String)authInfo.get("user");
            String authType = (String)authInfo.get("authType");
            String authPass = (String)authInfo.get("authPass");
            String privType = (String)authInfo.get("privType");
            String privPass = (String)authInfo.get("privPass");

            return snmpV3.get(ip, port, user, authType, authPass, privType, privPass, oids);

            /*
            switch (method) {
                case "validate":
                    return snmpV3.validate(ip, port, user, authType, authPass, privType, privPass);
                case "get":
                    return snmpV3.get(ip, port, user, authType, authPass, privType, privPass, oids);
                case "walk":
                    return snmpV3.walk(ip, port, user, authType, authPass, privType, privPass, oids[0]);
                default:
                    new Exception("Unknown SNMP command : " + method).printStackTrace();
            }
            */

        } catch (Exception e) {
            new Exception("An Exception happend : " + e.getMessage()).printStackTrace();
        } finally {
            if (snmp != null) {
                try {
                    snmp.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (transport != null) {
                try {
                    transport.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

}